from .compatability_utils import *
from abstract_utilities import get_media_types
from abstract_utilities.json_utils import get_create_json_data
from .file_utils import *
from moviepy.editor import *
import moviepy.editor as mp
from abstract_ocr.ocr_utils import extract_image_texts_from_directory
from .audio_utils import *
from .seo_utils import get_seo_data
from .text_tools import *

def if_none_get_default(obj=None, default=None):
    if obj is None:
        obj = default
    return obj

class VideoTextPipeline:
    def __init__(self, video_path=None, output_dir=None, output_path=None, new_filename=None, info_data=None, output_option=None, info_path=None, whisper_result_path=None, reencode=None, frame_interval=None, remove_phrases=None):
        self.info = {}
        self.remove_phrases = remove_phrases or []
        self.reencode = if_none_get_default(obj=reencode, default=False)
        self.video_path = video_path
        self.info["video_path"] = self.video_path
        self.video_id = generate_file_id(self.video_path)
        self.info["video_id"] = self.video_id
        
        # Set output_path, ensuring it’s unique
        self.output_path = get_path(self.video_path, output_dir=output_dir, output_path=output_path, new_filename=new_filename)
        if self.output_path == self.video_path:
            self.output_path = create_unique_filename(self.video_path)
        self.output_dir = os.path.dirname(self.output_path)
        self.info["info_dir"] = self.output_dir
        
        # Set output_option, defaulting to "copy"
        self.output_option = output_option or "copy"
        self.info["output_path"] = self.output_path
        
        self.info_path = info_path or os.path.join(self.output_dir, 'info.json')
        self.info["info_path"] = self.info_path
        self.whisper_result_path = whisper_result_path or os.path.join(self.output_dir, 'whisper_result.json')
        self.info["whisper_result_path"] = self.whisper_result_path
        self.thumbnails_directory = os.path.join(self.output_dir, 'thumbnails')
        self.info["thumbnails_directory"] = self.thumbnails_directory
        self.frame_interval = if_none_get_default(obj=frame_interval, default=1)
        self.original_dir = os.path.dirname(self.video_path)
        self.audio_path = os.path.join(self.output_dir, 'audio.wav')
        self.info['audio_path'] = self.audio_path
        self.is_optimized = is_video_optimized(self.video_path)
        
        # Initialize info_data with processed_steps
        self.info_data = self.get_info(info_path=self.info_path, data=self.info)
        self.info_data.setdefault("processed_steps", {
            "conversion": False,
            "audio_extraction": False,
            "frame_extraction": False,
            "ocr": False,
            "transcription": False,
            "text_processing": False,
            "seo_data": False
        })
        self.info_data["is_optimized"] = self.info_data.get("is_optimized", self.is_optimized)
        # Save initial info_data
        safe_dump_to_file(data=self.info_data, file_path=self.info_path)

    def get_info(self, info_path=None, data=None):
        data = data or {}
        self.info_path = info_path or self.info_path
        if not os.path.isfile(self.info_path):
            safe_dump_to_file(data=data, file_path=self.info_path)
        else:
            og_data = safe_load_from_file(self.info_path)
            og_data = og_data or {}
            og_data.update(data)
            safe_dump_to_file(data=og_data, file_path=self.info_path)
        return safe_load_from_file(self.info_path)

    def needs_conversion(self):
        if self.info_data["processed_steps"]["conversion"]:
            return self.info_data.get("video_text") is None
        
        if not self.is_optimized:
            optimize_video_for_safari(
                input_file=self.video_path,
                output_option=self.output_option,
                output_path=self.output_path,
                reencode=self.reencode
            )
        elif os.path.isfile(self.output_path):
            input_hash = compute_file_hash(self.video_path, chunk_size=8192)
            output_hash = compute_file_hash(self.output_path, chunk_size=8192)
            if input_hash == output_hash:
                self.info_data["processed_steps"]["conversion"] = True
                safe_dump_to_file(data=self.info_data, file_path=self.info_path)
                return
            self.final_output = handle_output_option(self.video_path, self.output_path, self.output_option)
        else:
            self.final_output = handle_output_option(self.video_path, self.output_path, self.output_option)
        
        self.info_data["is_optimized"] = True
        self.info_data["processed_steps"]["conversion"] = True
        safe_dump_to_file(data=self.info_data, file_path=self.info_path)
        return self.info_data.get("video_text") is None

    def extract_audio(self):
        if self.info_data["processed_steps"]["audio_extraction"]:
            return
        extract_audio_from_video(video_path=self.output_path, audio_path=self.audio_path)
        self.info_data["processed_steps"]["audio_extraction"] = True
        safe_dump_to_file(data=self.info_data, file_path=self.info_path)

    def needs_transcription(self):
        if self.info_data["processed_steps"]["transcription"]:
            return safe_load_from_file(self.whisper_result_path)
        
        whisper_result = {}
        if not os.path.isfile(self.whisper_result_path):
            whisper_result = transcribe_with_whisper_local(audio_path=self.audio_path)
            safe_dump_to_file(data=whisper_result, file_path=self.whisper_result_path)
        whisper_result = get_create_json_data(file_path=self.whisper_result_path, data=whisper_result)
        if not whisper_result.get('text') or not whisper_result.get('segments'):
            whisper_result = transcribe_with_whisper_local(audio_path=self.audio_path)
            safe_dump_to_file(data=whisper_result, file_path=self.whisper_result_path)
        
        self.info_data["processed_steps"]["transcription"] = True
        safe_dump_to_file(data=self.info_data, file_path=self.info_path)
        return whisper_result

    def needs_text(self, whisper_text=None):
        if self.info_data["processed_steps"]["text_processing"]:
            return self.info_data
        
        info_data = self.info_data
        whisper_text = whisper_text or get_whisper_text_data(info_data=info_data)
        summarizer = pipeline("summarization", model="facebook/bart-large-cnn")
        
        if info_data.get('description') is None:
            info_data['description'] = summarizer(whisper_text[:1000], max_length=100, min_length=30, do_sample=False)[0]['summary_text']
        if info_data.get('keywords') is None:
            info_data = refine_keywords(full_text=whisper_text, top_n=5, info_data=info_data)
        if info_data.get('summary') is None:
            info_data['summary'] = get_summary(full_text=whisper_text, max_words=200, max_length=100, min_length=50)
        
        self.info_data = info_data
        self.info_data["processed_steps"]["text_processing"] = True
        safe_dump_to_file(data=self.info_data, file_path=self.info_path)
        return self.info_data

    def extract_frames(self):
        if self.info_data["processed_steps"]["frame_extraction"]:
            return
        
        video_path = self.video_path
        directory = self.thumbnails_directory
        interval = self.frame_interval
        vid_id = self.video_id

        clip = VideoFileClip(video_path)
        duration = int(clip.duration)

        os.makedirs(directory, exist_ok=True)
        for t in range(0, duration, interval):
            frame_file = os.path.join(directory, f"{vid_id}_frame_{t}.jpg")
            if not os.path.isfile(frame_file):
                frame = clip.get_frame(t)
                import cv2, numpy as np
                cv2.imwrite(frame_file, cv2.cvtColor(np.array(frame), cv2.COLOR_RGB2BGR))
        
        self.info_data["processed_steps"]["frame_extraction"] = True
        safe_dump_to_file(data=self.info_data, file_path=self.info_path)

    def ocr_images(self, existing_texts=None):
        if self.info_data["processed_steps"]["ocr"]:
            return self.info_data.get("video_text")
        
        texts = extract_image_texts_from_directory(
            directory=self.thumbnails_directory,
            image_texts=existing_texts or [],
            remove_phrases=self.remove_phrases)
        
        self.info_data["video_text"] = texts
        self.info_data["processed_steps"]["ocr"] = True
        safe_dump_to_file(data=self.info_data, file_path=self.info_path)
        return texts

    def get_seo_data(self):
        if self.info_data["processed_steps"]["seo_data"]:
            return self.info_data
        
        self.info_data = get_seo_data(info_data=self.info_data, infos_dir=self.output_dir)
        self.info_data["processed_steps"]["seo_data"] = True
        safe_dump_to_file(data=self.info_data, file_path=self.info_path)
        return self.info_data

    def run(self):
        """Perform the full OCR pipeline and update info_data & JSON on disk."""
        # Step 1: Video conversion
        self.needs_conversion()
        
        # Step 2: Audio extraction
        self.extract_audio()
        
        # Step 3: Frame extraction
        self.extract_frames()
        
        # Step 4: OCR on frames
        self.ocr_images(existing_texts=self.info_data.get("video_text"))
        
        # Step 5: Transcription
        whisper_result = self.needs_transcription()
        
        # Step 6: Text processing (summary, keywords, description)
        self.needs_text(whisper_text=whisper_result.get('text'))
        
        # Step 7: SEO data
        self.get_seo_data()
        
        # Final save
        safe_dump_to_file(data=self.info_data, file_path=self.info["info_path"])
        return self.info_data



from keybert_utils.keybert_manager import KeywordManager
from summarizer_utils.summarizer_manager import SummarizerManager
kw_mgr = KeywordManager()
summ_mgr = SummarizerManager()

@app.route("/generate_keywords", methods=["POST"])
def generate_keywords():
    req = request
    info = load_existing_info_for(req)
    new_info = kw_mgr.run(req=req, info_data=info)
    return jsonify(new_info)



@app.route("/generate_summary", methods=["POST"])
def generate_summary():
    req = request
    info = load_existing_info_for(req)
    updated = summ_mgr.run(req=req, info_data=info)
    return jsonify(updated)



import moviepy.editor as mp
from moviepy.editor import *
from abstract_ocr import extract_image_texts_from_directory
def extract_video_frames(video_path,directory,video_id=None,frame_interval=None):
    frame_interval = frame_interval or 1
    video = VideoFileClip(video_path)
    duration = video.duration
    video_id = video_id or generate_file_id(video_path)
    for t in range(0, int(duration), frame_interval):
        frame_path = os.path.join(directory,f"{video_id}_frame_{t}.jpg")
        if not os.path.isfile(frame_path):
            frame = video.get_frame(t)
            cv2.imwrite(frame_path, cv2.cvtColor(np.array(frame), cv2.COLOR_RGB2BGR))
def analyze_video_text(video_path,
                       directory=None,
                       image_texts=None,
                       remove_phrases=None,
                       video_id=None,
                       frame_interval=None):
    if video_path == None or not video_path or not os.path.isfile(video_path):
        return image_texts
    remove_phrases=remove_phrases or []
    output_directory = directory or os.getcwd()
    
    extract_video_frames(video_path=video_path,
                          directory=directory,
                          frame_interval=frame_interval)
    image_texts = extract_image_texts_from_directory(directory=directory,
                                                     image_texts=image_texts,
                                                     remove_phrases=remove_phrases)
    
    return image_texts




from .summarizer_services import get_summary
from ...whisper_utils import get_whisper_text_data
from ..keybert_utils.keybert_calls import get_keybert_list
def get_summary_data(info_data=None,**kwargs):
    full_text = get_whisper_text_data(info_data=info_data)
    keywords = get_keybert_list(**info_data)
    result = get_summary(keywords=keywords,
                         full_text=full_text)
    return result
def get_summary_info_data(info_data=None,**kwargs):
    info_data = info_data or {}
    result = get_summary_data(info_data=info_data,**kwargs)
    info_data['summary'] = result
    return info_data



from typing import Dict, Any, Optional
from pathlib import Path
import os
import shutil
from datetime import datetime
import unicodedata
import re
import hashlib
from abstract_utilities import safe_read_from_json, safe_dump_to_file, get_logFile

# Constants
BASE_URL = "https://typicallyoutliers.com"
TEXT_DIR = '/var/www/typicallyoutliers/frontend/public/repository/text_dir/'
VIDEO_DIR = '/var/www/typicallyoutliers/frontend/public/repository/Video/'
DIR_LINKS = {TEXT_DIR: 'infos', VIDEO_DIR: 'videos'}
logger = get_logFile('video_utils')

class videoPathManager:
    def __init__(self):
        self.video_paths = []
        self.video_ids = os.listdir(TEXT_DIR) if os.path.exists(TEXT_DIR) else []
        self.video_directories = self.get_all_video_paths()

    def get_all_video_paths(self) -> Dict[str, Dict[str, str]]:
        video_directories = {
            video_id: {"directory": os.path.join(TEXT_DIR, video_id)}
            for video_id in self.video_ids
        }
        for video_id in video_directories:
            info_path = os.path.join(video_directories[video_id]["directory"], 'info.json')
            info_data = safe_read_from_json(info_path) or {}
            video_directories[video_id]['video_path'] = info_data.get('video_path', '')
        return video_directories

    def get_video_path(self, video_id: str) -> Optional[str]:
        return self.video_directories.get(video_id, {}).get('video_path')

    def get_video_id(self, video_path: str) -> Optional[str]:
        for video_id, values in self.video_directories.items():
            if video_path == values.get('video_path'):
                return video_id
        video_id = generate_video_id(video_path)
        self.video_ids.append(video_id)
        self.add_path(video_id, video_path)
        return video_id

    def add_path(self, video_id: str, video_path: str) -> Dict[str, str]:
        if video_id not in self.video_directories:
            self.video_directories[video_id] = {}
        self.video_directories[video_id]['video_path'] = video_path
        self.video_directories[video_id]["directory"] = os.path.join(TEXT_DIR, video_id)
        return self.video_directories[video_id]

# Instantiate videoPathManager
video_path_mgr = videoPathManager()

def generate_video_id(path: str, max_length: int = 50) -> str:
    base = os.path.splitext(os.path.basename(path))[0]
    base = unicodedata.normalize('NFKD', base).encode('ascii', 'ignore').decode('ascii')
    base = base.lower()
    base = re.sub(r'[^a-z0-9]+', '-', base).strip('-')
    base = re.sub(r'-{2,}', '-', base)
    if len(base) > max_length:
        h = hashlib.sha1(base.encode()).hexdigest()[:8]
        base = base[:max_length - len(h) - 1].rstrip('-') + '-' + h
    return base

def get_link(path: Optional[str]) -> Optional[str]:
    if path:
        for key, value in DIR_LINKS.items():
            if path.startswith(key):
                rel_path = path[len(key):]
                return f"{BASE_URL}/{value}/{rel_path}"
    return None

def get_complete_video_data(video_path: Optional[str] = None, video_id: Optional[str] = None, force_refresh: bool = False) -> Dict[str, Any]:
    """
    Efficiently retrieve all available video data, minimizing resource usage by reusing existing data.
    Returns a unified dictionary with video metadata, transcription, thumbnails, and SEO data.

    Args:
        video_path (str, optional): Absolute path to the video file.
        video_id (str, optional): Unique identifier for the video (from TEXT_DIR).
        force_refresh (bool): If True, reprocess all data (e.g., retranscribe audio).

    Returns:
        Dict[str, Any]: Comprehensive video data, with defaults for missing fields.
    """
    # Initialize default response
    video_data = {
        "id": None,
        "title": "Untitled Video",
        "embed": "",
        "description": "Check out this video",
        "keywords_str": "",
        "thumbnail_url": "/var/www/typicallyoutliers/frontend/public/imgs/no_image.jpg",
        "contentUrl": "",
        "video_url": "",
        "optimized_video_url": "",
        "ext": "",
        "mime_type": "video/mp4",
        "category": "Education",
        "transcript": "",
        "captions": None,
        "schema_markup": {},
        "social_meta": {},
        "publication_date": datetime.now().strftime("%Y-%m-%d"),
        "file_metadata": {},
        "thumbnail_metadata": {},
        "audio_text": [],
        "error": None
    }

    try:
        # Step 1: Resolve video_id and info_data
        info_data = {}
        if video_id:
            info_path = os.path.join(TEXT_DIR, video_id, 'info.json')
            info_data = safe_read_from_json(info_path) or {}
        elif video_path:
            video_id = video_path_mgr.get_video_id(video_path)
            if not video_id:
                video_id = generate_video_id(video_path)
                video_dir = os.path.join(VIDEO_DIR, video_id)
                new_video_path = os.path.join(video_dir, os.path.basename(video_path))
                if video_path != new_video_path and os.path.isfile(video_path):
                    os.makedirs(video_dir, exist_ok=True)
                    shutil.move(video_path, new_video_path)
                    video_path = new_video_path
                video_path_mgr.add_path(video_id, video_path)
            info_path = os.path.join(TEXT_DIR, video_id, 'info.json')
            info_data = safe_read_from_json(info_path) or {}

        if not video_id:
            video_data["error"] = "Either video_path or video_id must be provided"
            return video_data

        # Initialize info_data if empty
        if not info_data:
            info_data = {
                "video_id": video_id,
                "video_path": video_path or video_path_mgr.get_video_path(video_id),
                "info_path": os.path.join(TEXT_DIR, video_id, "info.json"),
                "info_directory": os.path.join(TEXT_DIR, video_id),
                "publication_date": datetime.now().strftime("%Y-%m-%d")
            }
            os.makedirs(info_data["info_directory"], exist_ok=True)

        # Step 2: Populate video data with existing or default values
        video_data["id"] = video_id
        video_data["video_url"] = info_data.get("video_url", get_link(info_data.get("video_path", "")) or "")
        video_data["contentUrl"] = video_data["video_url"]
        video_data["optimized_video_url"] = video_data["video_url"]
        video_data["publication_date"] = info_data.get("publication_date", datetime.now().strftime("%Y-%m-%d"))
        video_data["audio_text"] = info_data.get("audio_text", [])
        video_data["transcript"] = info_data.get("transcript", "")
        video_data["description"] = info_data.get("seo_description", "Check out this video")
        video_data["title"] = info_data.get("seo_title", "Untitled Video")
        video_data["keywords_str"] = info_data.get("keywords_str", "")
        video_data["schema_markup"] = info_data.get("schema_markup", {})
        video_data["social_meta"] = info_data.get("social_meta", {})
        video_data["file_metadata"] = info_data.get("file_metadata", {})
        video_data["thumbnail_metadata"] = info_data.get("thumbnail_metadata", {})
        video_data["thumbnail_url"] = info_data.get("thumbnail_url", "/var/www/typicallyoutliers/frontend/public/imgs/no_image.jpg")
        video_data["captions"] = info_data.get("captions", None)

        # Set extension and mime type
        if video_data["video_url"]:
            _, ext = os.path.splitext(os.path.basename(video_data["video_url"]))
            video_data["ext"] = ext
            video_data["mime_type"] = "video/mp4" if ext == ".mp4" else "video/webm" if ext == ".webm" else "video/unknown"

        # Step 3: Save updated info_data if changed
        if any(
            info_data.get(key) != video_data.get(key)
            for key in ["video_url", "audio_text", "transcript", "thumbnail_url", "file_metadata", "thumbnail_metadata", "schema_markup", "social_meta"]
        ):
            safe_dump_to_file(info_data, info_data["info_path"])

    except Exception as e:
        logger.error(f"Error processing video {video_id or video_path}: {str(e)}")
        video_data["error"] = str(e)

    return video_data
video_path = '/home/computron/mnt/webserver/typicallyoutliers/backend/typicallyoutliers_flask/functions/videos/Charlotte Gerson Interviewed on Cancer & Medical Industry Fraud 1_5/Charlotte Gerson Interviewed on Cancer & Medical Industry Fraud 1_5.mp4'

data = get_complete_video_data(video_path)
input(data)



from .routes import *
import numpy as np
from typing import *
from pydub import AudioSegment
from pydub.silence import split_on_silence
def chunk_fixed(
    audio_array: np.ndarray,
    sr: int = 16000,
    chunk_length_s: int = 30,
    overlap_s: int = 5
) -> List[np.ndarray]:
    """
    Split a numpy audio array into fixed-size chunks with overlap.
    """
    chunk_len = chunk_length_s * sr
    overlap  = overlap_s  * sr
    step     = chunk_len - overlap
    total    = audio_array.shape[0]
    chunks: List[np.ndarray] = []
    for start in range(0, total, step):
        end = min(start + chunk_len, total)
        chunks.append(audio_array[start:end])
        if end == total:
            break
    return chunks

def chunk_on_silence(
    audio_path: str,
    min_silence_len: int = 700,
    silence_thresh: Optional[int] = None,
    keep_silence: int = 300
) -> List[AudioSegment]:
    """
    Load the file and split on silent parts.
    Returns a list of pydub AudioSegment chunks.
    """
    audio = AudioSegment.from_file(audio_path)
    # dynamically set threshold ~16 dB below average loudness
    silence_thresh = silence_thresh or int(audio.dBFS) - 16
    return split_on_silence(
        audio,
        min_silence_len=min_silence_len,
        silence_thresh=silence_thresh,
        keep_silence=keep_silence
    )
def transcribe_in_chunks(
    audio_path: str,
    model_size: str      = "medium",
    language: Optional[str] = None,
    use_silence: bool    = True,
    info_data = None,
    # parameters for fixed chunking:
    chunk_length_s: int  = 30,
    overlap_s: int       = 5,
    # parameters for silence chunking:
    min_silence_len: int = 700,
    keep_silence: int    = 300
) -> str:
    """
    Transcribe by splitting the audio into chunks and stitching results.
    """
    info_data =info_data or {}
    model = whisper.load_model(model_size)
    audio_dir =  make_audio_dir(info_data)
    
    full_text = []
    if use_silence:
        segments = chunk_on_silence(
            audio_path,
            min_silence_len=min_silence_len,
            keep_silence=keep_silence
        )
        audio_paths  = []
        for i, seg in enumerate(segments):
            audio_path = os.path.join(audio_dir,f"chunk_{i}.wav")
            audio_paths.append(audio_path)
            seg.export(audio_path, format="wav")
            res = model.transcribe(audio_path, language=language)
            full_text.append(res["text"].strip())
            remove_path(path=audio_path)
        remove_directory(audio_dir,paths=audio_paths)
    else:
        audio = whisper.load_audio(audio_path)  # returns np.ndarray
        # no need to pad/trim if chunking manually
        chunks = chunk_fixed(
            audio, SAMPLE_RATE,
            chunk_length_s=chunk_length_s,
            overlap_s=overlap_s
        )
        full_text = collate_trans_chunks(chunks=chunks,
                                                 transcribe_func = model.transcribe,
                                                 key="text",
                                                 full_text=full_text,
                                                 language=language)
    result = {"text":" ".join(full_text).strip()}
    info_data["whisper_result"] = result
    return info_data



from .functions import (os,
                        logger,
                        whisper,
                        AudioSegment,
                        sr,
                        detect_nonsilent,
                        format_timestamp,
                        safe_dump_to_file,
                        List,
                        shutil)

SAMPLE_RATE = whisper.audio.SAMPLE_RATE  # 16000 Hz
def transcribe_audio_file_clean(
    audio_path: str,
    json_data: str = None,
    min_silence_len: int = 500,
    silence_thresh_delta: int = 16
    ):
    """
    Load `audio_path`, detect all non-silent ranges, transcribe each,
    and (optionally) dump to JSON at `output_json`.
    """
    recognizer = sr.Recognizer()
    audio = AudioSegment.from_file(audio_path)

    # 1) Calibrate once on the first second
    calib = audio[:1000]
    calib_path = os.path.join(os.path.dirname(audio_path), "_calib.wav")
    calib.export(calib_path, format="wav")
    with sr.AudioFile(calib_path) as src:
        recognizer.adjust_for_ambient_noise(src, duration=1)
    os.remove(calib_path)

    # 2) Compute dynamic silence threshold, then find real speech segments
    silence_thresh = audio.dBFS - silence_thresh_delta
    nonsilent = detect_nonsilent(
        audio,
        min_silence_len=min_silence_len,
        silence_thresh=silence_thresh
    )
    
    json_data["audio_text"] = []
    for idx, (start_ms, end_ms) in enumerate(nonsilent):
        logger.info(f"Transcribing segment {idx}: {start_ms}-{end_ms} ms")
        chunk = audio[start_ms:end_ms]

        chunk_path = f"_chunk_{idx}.wav"
        chunk.export(chunk_path, format="wav")

        with sr.AudioFile(chunk_path) as src:
            audio_data = recognizer.record(src)
        try:
            text = recognizer.recognize_google(audio_data)
        except sr.UnknownValueError:
            text = ""

        json_data["audio_text"].append({
            "start_time": format_timestamp(start_ms),
            "end_time": format_timestamp(end_ms),
            "text": text
        })
        os.remove(chunk_path)

    # 3) Optionally write out the JSON

        full_text = [ entry["text"] 
                for entry in json_data.get("audio_text", []) 
                if entry.get("text") ]
        full_text = " ".join(full_text).strip()
        json_data["full_text"] = full_text
        safe_dump_to_file(json_data, json_data['info_path'])
    
    return json_data

def chunk_fixed(
    audio_array: np.ndarray,
    sr: int = SAMPLE_RATE,
    chunk_length_s: int = 30,
    overlap_s: int = 5
) -> List[np.ndarray]:
    """
    Split a numpy audio array into fixed-size chunks with overlap.
    """
    chunk_len = chunk_length_s * sr
    overlap  = overlap_s  * sr
    step     = chunk_len - overlap
    total    = audio_array.shape[0]
    chunks: List[np.ndarray] = []
    for start in range(0, total, step):
        end = min(start + chunk_len, total)
        chunks.append(audio_array[start:end])
        if end == total:
            break
    return chunks

def chunk_on_silence(
    audio_path: str,
    min_silence_len: int = 700,
    silence_thresh: Optional[int] = None,
    keep_silence: int = 300
) -> List[AudioSegment]:
    """
    Load the file and split on silent parts.
    Returns a list of pydub AudioSegment chunks.
    """
    audio = AudioSegment.from_file(audio_path)
    # dynamically set threshold ~16 dB below average loudness
    silence_thresh = silence_thresh or int(audio.dBFS) - 16
    return split_on_silence(
        audio,
        min_silence_len=min_silence_len,
        silence_thresh=silence_thresh,
        keep_silence=keep_silence
    )

def transcribe_in_chunks(
    audio_path: str,
    model_size: str      = "medium",
    language: Optional[str] = None,
    use_silence: bool    = True,
    info_data = None,
    # parameters for fixed chunking:
    chunk_length_s: int  = 30,
    overlap_s: int       = 5,
    # parameters for silence chunking:
    min_silence_len: int = 700,
    keep_silence: int    = 300
) -> str:
    """
    Transcribe by splitting the audio into chunks and stitching results.
    """
    info_data =info_data or {}
    model = whisper.load_model(model_size)
    output_dir = info_data.get('info_dir') or info_data.get('info_directory') or info_data.get('output_directory') or info_data.get('directory') or os.getcwd()
    audio_dir = os.path.join(output_dir,'audio')
    os.makedirs(audio_dir,exist_ok=True)
    full_text = []
    if use_silence:
        segments = chunk_on_silence(
            audio_path,
            min_silence_len=min_silence_len,
            keep_silence=keep_silence
        )
        audio_paths  = []
        for i, seg in enumerate(segments):
            audio_path = os.path.join(audio_dir,f"chunk_{i}.wav")
            audio_paths.append(audio_path)
            seg.export(audio_path, format="wav")
            res = model.transcribe(audio_path, language=language)
            full_text.append(res["text"].strip())
            if audio_path and os.path.isfile(audio_path):
                os.remove(audio_path)
        shutil.rmtree(audio_dir)
        for audio_path in audio_paths:
            if audio_path and os.path.isfile(audio_path):
                os.remove(audio_path)
        
    else:
        audio = whisper.load_audio(audio_path)  # returns np.ndarray
        # no need to pad/trim if chunking manually
        chunks = chunk_fixed(
            audio, SAMPLE_RATE,
            chunk_length_s=chunk_length_s,
            overlap_s=overlap_s
        )
        for chunk in chunks:
            res = model.transcribe(chunk, language=language)
            full_text.append(res["text"].strip())
    result = {"text":" ".join(full_text).strip()}
    info_data["whisper_result"] = result
    return info_data
def transcribe_with_whisper_local(
    audio_path: str,
    model_size: str = "tiny",           # one of "tiny", "base", "small", "medium", "large"
    language: str = "english",
    use_silence=False,
    info_data=None):
    info_data =info_data or {}
    # parameters for fixed chunki)
    """
    Returns the full transcript as a string.
    """
    model = whisper.load_model(model_size)           # loads to GPU if available
    # options: you can pass `task="translate"` for translating to English
 
    result = model.transcribe(audio_path, language=language)
    info_data["whisper_result"] = result
    return info_data



from ..routes import *
def eatAllQuotes(string):
    return eatAll(string,['"',"'",'`']).replace('" "',' ')
def chunk_summaries(chunks, max_length=None, min_length=None, truncation=True):  # Enabled truncation by default
    max_length = max_length or 100  # Reduced max_length
    min_length = min_length or 50  # Increased min_length for balance
    summaries = []
    for idx, chunk in enumerate(chunks):
        out = summarizer(
            chunk,
            max_length=max_length,
            min_length=min_length,
            truncation=truncation
        )
        summaries.append(out[0]["summary_text"])
    return summaries

def split_to_chunk(full_text, max_words=None):
    max_words = max_words or 200  # Reduced max_words
    sentences = full_text.split(". ")
    chunks, buf = [], ""
    for sent in sentences:
        if len((buf + sent).split()) <= max_words:
            buf += sent + ". "
        else:
            chunks.append(buf.strip())
            buf = sent + ". "
    if buf:
        chunks.append(buf.strip())
    return chunks
def get_summary(
    full_text,
    keywords=None,
    max_words=None,
    max_length=None,
    min_length=None,
    truncation=True
):
    summary = None
    if full_text and summarizer:
        chunks = split_to_chunk(full_text, max_words=max_words)
        summaries = chunk_summaries(
            chunks,
            max_length=max_length,
            min_length=min_length,
            truncation=truncation
        )
        # Stitch and enforce total word limit
        summary = " ".join(summaries).strip()
        # Post-process to limit total summary to 150 words
        words = summary.split()
        if len(words) > 150:
            summary = ' '.join(words[:150]) + "..."
    return summary



from ..routes import *
import spacy
from collections import Counter

nlp = spacy.load("en_core_web_sm")

def get_keybert(full_text, keyphrase_ngram_range=None, top_n=None, stop_words=None, use_mmr=None, diversity=None):
    keyphrase_ngram_range = keyphrase_ngram_range or (1, 2)  # Reduced max n-gram range
    top_n = top_n or 5  # Reduced top_n
    stop_words = stop_words or "english"
    use_mmr = use_mmr or True
    diversity = diversity or 0.6  # Increased diversity for less overlap
    keybert = kw_model.extract_keywords(
        full_text,
        keyphrase_ngram_range=keyphrase_ngram_range,
        stop_words=stop_words,
        top_n=top_n * 2,  # Extract more initially for filtering
        use_mmr=use_mmr,
        diversity=diversity
    )
    # Filter by score threshold (e.g., > 0.3) to keep high-relevance keywords
    filtered_keybert = [(kw, score) for kw, score in keybert if score > 0.3][:top_n]
    return filtered_keybert
def extract_keywords_nlp(text, top_n=5):  # Reduced top_n
    doc = nlp(str(text))
    if not isinstance(text, str):
        logger.info(f"this is not a string: {text}")
    # Focus on NOUN, PROPN, and multi-word entities, stricter length filter
    word_counts = Counter(token.text.lower() for token in doc 
                          if token.pos_ in ["NOUN", "PROPN"] and not token.is_stop and len(token.text) > 3)
    entity_counts = Counter(ent.text.lower() for ent in doc.ents 
                            if len(ent.text.split()) >= 2 and ent.label_ in ["PERSON", "ORG", "GPE", "EVENT"])
    # Combine and take top 5, prioritize entities
    top_keywords = [word for word, _ in (entity_counts + word_counts).most_common(top_n)]
    return top_keywords

def refine_keywords(full_text=None, keywords=None, keyphrase_ngram_range=None, top_n=None, stop_words=None, use_mmr=None, diversity=None, info_data={}):
    top_n = top_n or 5
    keywords = keywords or extract_keywords_nlp(full_text, top_n=top_n)
    keybert = get_keybert(full_text, keyphrase_ngram_range=keyphrase_ngram_range, top_n=top_n, stop_words=stop_words, use_mmr=use_mmr, diversity=diversity)
    combined_keywords = list(set(kw.lower() for kw, _ in keybert) | set(kw.lower() for kw in keywords))[:top_n]
    keyword_density = calculate_keyword_density(full_text, combined_keywords)
    if info_data:
        info_data['keywords'], info_data['combined_keywords'], info_data['keyword_density'] = keywords, combined_keywords, keyword_density
        return info_data
    return keywords, combined_keywords, keyword_density
import re

def calculate_keyword_density(text, keywords):
    if not text or not keywords:
        return {kw: 0.0 for kw in keywords} if keywords else {}
    # Split text into words, removing punctuation
    words = [word.strip(".,!?").lower() for word in re.split(r'\s+', text) if word.strip(".,!?")]
    total_words = len(words)
    if total_words == 0:
        return {kw: 0.0 for kw in keywords}
    return {kw: (words.count(kw.lower()) / total_words) * 100 for kw in keywords if kw}



from .routes import *

# whisper_calls.py
from .whisper_pipeline import WhisperPipeline
from .routes import safe_read_from_json, get_result_from_data

def get_whisper_result_data(**kwargs):
    return WhisperPipeline(kwargs).run()

def get_whisper_text(**kwargs):
    # still called get_whisper_text, but now driven by the pipeline
    data = get_whisper_result_data(**kwargs)
    return get_result_from_data("text", lambda **k: data, **kwargs)

def get_whisper_segment(**kwargs):
    data = get_whisper_result_data(**kwargs)
    return get_result_from_data("segments", lambda **k: data, **kwargs)

def get_recieve_whisper_data(data, **kwargs):
    # you likely won’t need this anymore, pipeline.save() covers it
    return safe_save_updated_json_data(
        data,
        WhisperPipeline(kwargs)._result_path(),
        valid_keys=VALID_KEYS,
        invalid_keys=INVALID_KEYS,
    )

def is_whisper_data(**info_data):
    return WhisperPipeline(info_data).exists()



from .routes import *
logger =get_logFile() 
nlp = spacy.load("en_core_web_sm")
def get_content_length(text):
    for each in ['into a ']:
        if each in text:
            text = text.split(each)[1]
            break
    for each in [' word']:
        if each in text:
            text = text.split(each)[0]
            break
    numbers = []
    for each in text.split('-'):
        numbers.append('')
        for char in each:
            char = str(char)
            if char in list('1234567890'):
                numbers[-1]+=char
    for i,number in enumerate(numbers):
        if number:
            numbers[i]=int(number)*10
    return numbers
    
def refine_with_gpt(full_text: str,task=None) -> str:
    prompt = generate_with_bigbird(full_text,task=task)
    lengths = get_content_length(full_text)
    max_length=200
    min_length=100
    if lengths:
        max_l = lengths[-1]
        if max_l:   
            max_length = int(max_l)
        min_l = lengths[0]
        if min_l: 
            min_length = int(min_l)
    out = generator(prompt, min_length=min_length,max_length=max_length, num_return_sequences=1)[0]["generated_text"]
    return out.strip()
def generate_with_bigbird(text: str, task: str = "title", model_dir: str = "allenai/led-base-16384") -> str:
    try:
        tokenizer = LEDTokenizer.from_pretrained(model_dir)
        model = LEDForConditionalGeneration.from_pretrained(model_dir)
        prompt = (
            f"Generate a concise, SEO-optimized {task} for the following content: {text[:1000]}"
            if task in ["title", "caption", "description"]
            else f"Summarize the following content into a 100-150 word SEO-optimized abstract: {text[:4000]}"
        )
        inputs = tokenizer(prompt, return_tensors="pt", max_length=512, truncation=True)
        outputs = model.generate(
            inputs["input_ids"],
            max_length=200 if task in ["title", "caption"] else 300,
            num_beams=5,
            early_stopping=True
        )
        return tokenizer.decode(outputs[0], skip_special_tokens=True)
    except Exception as e:
        print(f"Error in BigBird processing: {e}")
        return ""



import moviepy.editor as mp
from moviepy.editor import *
from.file_utils import logger
import whisper
def extract_audio_from_video(video_path, audio_path=None):
    """Extract audio from a video file using moviepy."""
    if audio_path == None:
        video_directory = os.path.dirname(video_path)
        audio_path = video_directory
    if os.path.isdir(audio_path):
        audio_path = os.path.join(audio_path,'audio.wav')
    try:
        logger.info(f"Extracting audio from {video_path} to {audio_path}")
        video = mp.VideoFileClip(video_path)
        video.audio.write_audiofile(audio_path)
        video.close()
        if not os.path.exists(audio_path):
            raise FileNotFoundError(f"Audio file {audio_path} was not created.")
        logger.info(f"Audio extracted successfully: {audio_path}")
        return audio_path
    except Exception as e:
        logger.error(f"Error extracting audio from {video_path}: {e}")
        return None
def transcribe_with_whisper_local(
    audio_path: str,
    model_size: str = "tiny",           # one of "tiny", "base", "small", "medium", "large"
    language: str = "english",
    use_silence=True):
    # parameters for fixed chunki)
    """
    Returns the full transcript as a string.
    """
    model = whisper.load_model(model_size)           # loads to GPU if available
    # options: you can pass `task="translate"` for translating to English
 
    result = model.transcribe(audio_path, language=language)
    return result



from .routes import *
def get_data(key,**info_data):
    return info_data.get(key)



def get_keybert_list(**info_data):
    keybert_keywords_bool = is_keybert_keywords_data(**info_data)
    keybert_combined_keywords_bool = is_keybert_combined_keywords_data(**info_data)
    keybert_bool_list = [keybert_keywords_bool,keybert_combined_keywords_bool]
    keybert_bool_key_list = ['keywords','combined_keywords']
    keybert_all_list = []
    for i,keybert_bool in enumerate(keybert_bool_list):
        if get_keybert_keywords_data(**info_data):
            keybert_key = keybert_bool_key_list[i]
            key_data = get_data(keybert_key,**info_data)
            if key_data:
                keybert_all_list += key_data
    return keybert_all_list

def is_keybert_data(**info_data):
    keybert_keywords_data = get_keybert_list(**info_data)
    return (keybert_keywords_data in [[],'',"",None])


def get_keybert_keywords_data(**info_data):
    return get_data('keywords',**info_data)

def is_keybert_keywords_data(**info_data):
    keybert_keywords = get_keybert_keywords_data(**info_data)
    bool_key = (keybert_keywords == None)
    return bool_key


def get_keybert_combined_keywords_data(**info_data):
    return get_data('combined_keywords',**info_data)

def is_keybert_combined_keywords_data(**info_data):
    keybert_combined_keywords = get_keybert_combined_keywords_data(**info_data)
    bool_key = (keybert_combined_keywords == None)
    return bool_key



#!/usr/bin/env python3
import os
import glob
import logging
import json
import os,sys,spacy
from multiprocessing import Process
import speech_recognition as sr
from abstract_utilities import *
from moviepy.editor import *
import moviepy.editor as mp
from pydub import AudioSegment
from datetime import timedelta
from .video_utils import derive_all_video_meta

logger = get_logFile('vid_to_aud')

# Set up logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    handlers=[
        logging.FileHandler('vid_to_aud.log'),
        logging.StreamHandler()
    ]
)

# Initialize recognizer
r = sr.Recognizer()




def create_key_value(json_obj,key,value):
    if key not in json_obj:
        json_obj[key]=value
    return json_obj
def transcribe_audio_file(audio_path, json_data, chunk_length_ms=60000,summarizer=None):
    """Transcribe audio file in chunks and save as text and time-blocked JSON."""
    try:
        logging.info(f"Transcribing audio: {audio_path}")
        audio = AudioSegment.from_wav(audio_path)
        chunks = [audio[i:i + chunk_length_ms] for i in range(0, len(audio), chunk_length_ms)]
        full_text = ""
        json_data = create_key_value(json_data,'audio_text',[])
        file_path = json_data['info_path']
        for i, chunk in enumerate(chunks):
            start_time = i * chunk_length_ms
            end_time = min((i + 1) * chunk_length_ms, len(audio))
            chunk_path = f"chunk_{i}.wav"
            chunk.export(chunk_path, format="wav")
            logging.info(f"Processing chunk {i+1}/{len(chunks)} ({format_timestamp(start_time)} - {format_timestamp(end_time)})")
        
            with sr.AudioFile(chunk_path) as source:
                r.adjust_for_ambient_noise(source)
                audio_data = r.record(source)
                try:
                    text = r.recognize_google(audio_data)
                    full_text += text + " "
                    json_data['audio_text'].append({
                        "start_time": format_timestamp(start_time),
                        "end_time": format_timestamp(end_time),
                        "text": text
                    })
                except sr.UnknownValueError:
                    logging.warning(f"Chunk {i+1} could not be transcribed.")
                    json_data['audio_text'].append({
                        "start_time": format_timestamp(start_time),
                        "end_time": format_timestamp(end_time),
                        "text": ""
                    })
                except sr.RequestError as e:
                    logging.error(f"API error for chunk {i+1}: {e}")
                    json_data['audio_text'].append({
                        "start_time": format_timestamp(start_time),
                        "end_time": format_timestamp(end_time),
                        "text": ""
                    })
                safe_dump_to_file(data=json_data,file_path=json_path) 
            os.remove(chunk_path)  # Clean up chunk file

        # Save plain text
        if full_text:
            full_text = full_text.strip()
        text_result= get_voice(full_text, text=full_text)
        json_data = create_key_value(json_data,'full_text',text_result)
        json_data = create_key_value(json_data,'summary',None)
        json_data = create_key_value(json_data,'keywords',extract_keywords_nlp(text_result))
        if summarizer:
            try:
                json_data['summary'] = summarizer(text_result, max_length=160, min_length=40)
            except Exception as e:
                logging.error(f"Error getting summary for {json_data['filename']}: {e}")
        
        return json_data
    except Exception as e:
        logging.error(f"Error transcribing {audio_path}: {e}")
        return json_data





def initiate_process(target,*args):
    p = Process(target=target, args=args)
    p.start()
    logging.info(f"Started process for: {args}")
def transcribe_all_video_paths(directory=None,output_dir=None,remove_phrases=None,summarizer=None,get_vid_data=None):
    get_vid_data = get_vid_data or False
    remove_phrases=remove_phrases or []
    directory = directory or os.getcwd()
    output_dir = output_dir if output_dir else make_dirs(directory,'text_dir')
    paths = glob.glob(path_join(directory, '**', '**'), recursive=True)
    paths = [file_path for file_path in paths if confirm_type(file_path,media_types=get_media_types(['video']))]
    video_paths = get_all_file_types(directory=directory,types='video') or get_all_file_types(directory=abs_dirname,types='videos')
    for video_path in video_paths:
        info = get_info_data(video_path,output_dir=output_dir,remove_phrases=remove_phrases)
        extract_audio_from_video(video_path=info['video_path'],audio_path=info['audio_path'])
        info = transcribe_audio_file(audio_path=info['audio_path'],json_data=info,summarizer=summarizer,get_vid_data=get_vid_data)
        safe_dump_to_file(data=info,file_path=info['info_path']) 
def get_info_data(video_path,output_dir=None,remove_phrases=None):
    
    remove_phrases=remove_phrases or []
    dirname = os.path.dirname(video_path)
    basename = os.path.basename(video_path)
    filename,ext = os.path.splitext(basename)
    video_directory = make_dirs(output_dir,filename)
    info_path = os.path.join(video_directory,'info.json')
    video_text_path = os.path.join(video_directory,'video_text.json')
    audio_path = os.path.join(video_directory,'audio.wav')
    video_json_path = os.path.join(video_directory,'video_json.json')
    info = {}
    if os.path.isfile(info_path):
        info = safe_read_from_file(info_path)
    info['video_path']=video_path
    info['info_directory']=video_directory
    info['info_path']=info_path
    info['filename']=filename
    info['ext']=ext
    info['remove_phrases']=remove_phrases
    info['audio_path']=audio_path
    info['video_json']=video_json_path
    safe_dump_to_file(data=info,file_path=info['info_path'])
    return info
       
    



from .keybert_services import refine_keywords
from ...whisper_utils import *
def get_keywords_data(info_data=None):
    text_data = get_whisper_text_data(info_data=info_data)
    keywords = get_keywords_list(info_data=info_data)
    keywords,combined_keywords,keyword_density = refine_keywords(
        keywords=keywords,
        full_text=full_text)
    return keywords,combined_keywords,keyword_density
def get_keywords_info_data(info_data=None,**kwargs):
    info_data = info_data or {}
    keywords,combined_keywords,keyword_density = get_keywords_data(info_data=info_data,**kwargs)
    info_data['keywords'] = keywords
    info_data['combined_keywords'] = combined_keywords
    info_data['keyword_density'] = keyword_density
    return info_data



#!/usr/bin/env python3
import os
import glob
import logging
import json
import os,sys,spacy
from multiprocessing import Process
import speech_recognition as sr
from abstract_utilities import *
from moviepy.editor import *
import moviepy.editor as mp
from pydub import AudioSegment
from datetime import timedelta
from .video_utils import derive_all_video_meta
from collections import Counter
r = sr.Recognizer()
logger = get_logFile('vid_to_aud')
nlp = spacy.load("en_core_web_sm")
def format_timestamp(ms):
    """Convert milliseconds to a formatted timestamp (HH:MM:SS.mmm)."""
    td = timedelta(milliseconds=ms)
    hours, remainder = divmod(td.seconds, 3600)
    minutes, seconds = divmod(remainder, 60)
    milliseconds = td.microseconds // 1000
    return f"{hours:02d}:{minutes:02d}:{seconds:02d}.{milliseconds:03d}"

def extract_audio_from_video(video_path, audio_path):
    """Extract audio from a video file using moviepy."""
    try:
        logging.info(f"Extracting audio from {video_path} to {audio_path}")
        video = mp.VideoFileClip(video_path)
        video.audio.write_audiofile(audio_path)
        video.close()
        if not os.path.exists(audio_path):
            raise FileNotFoundError(f"Audio file {audio_path} was not created.")
        logging.info(f"Audio extracted successfully: {audio_path}")
        return audio_path
    except Exception as e:
        logging.error(f"Error extracting audio from {video_path}: {e}")
        return None
def create_key_value(json_obj,key,value):
    if key not in json_obj:
        json_obj[key]=value
    return json_obj
def check_if_in_time(json_data,start_time,end_time):
    audio_text = json_data.get('audio_text')
    for key,values in audio_text.items():
        value_start_time = values.get("start_time")
        value_end_time = values.get("end_time")
        if value_end_time - value_start_time
def transcribe_audio_file(audio_path, json_data, chunk_length_ms=60000,summarizer=None):
    """Transcribe audio file in chunks and save as text and time-blocked JSON."""
    try:
        logging.info(f"Transcribing audio: {audio_path}")
        audio = AudioSegment.from_wav(audio_path)
        chunks = [audio[i:i + chunk_length_ms] for i in range(0, len(audio), chunk_length_ms)]
        full_text = ""
        json_data = create_key_value(json_data,'audio_text',[])
        file_path = json_data['info_path']
        for i, chunk in enumerate(chunks):
            start_time = i * chunk_length_ms
            end_time = min((i + 1) * chunk_length_ms, len(audio))
            chunk_path = f"chunk_{i}.wav"
            chunk.export(chunk_path, format="wav")
            logging.info(f"Processing chunk {i+1}/{len(chunks)} ({format_timestamp(start_time)} - {format_timestamp(end_time)})")
            
            with sr.AudioFile(chunk_path) as source:
                r.adjust_for_ambient_noise(source)
                audio_data = r.record(source)
                try:
                    text = r.recognize_google(audio_data)
                    full_text += text + " "
                    json_data['audio_text'].append({
                        "start_time": format_timestamp(start_time),
                        "end_time": format_timestamp(end_time),
                        "text": text
                    })
                except sr.UnknownValueError:
                    logging.warning(f"Chunk {i+1} could not be transcribed.")
                    json_data['audio_text'].append({
                        "start_time": format_timestamp(start_time),
                        "end_time": format_timestamp(end_time),
                        "text": ""
                    })
                except sr.RequestError as e:
                    logging.error(f"API error for chunk {i+1}: {e}")
                    json_data['audio_text'].append({
                        "start_time": format_timestamp(start_time),
                        "end_time": format_timestamp(end_time),
                        "text": ""
                    })
                safe_dump_to_file(data=json_data,file_path=json_path) 
            os.remove(chunk_path)  # Clean up chunk file

        # Save plain text
        if full_text:
            full_text = full_text.strip()
        text_result= get_voice(full_text, text=full_text)
        json_data = create_key_value(json_data,'full_text',text_result)
        json_data = create_key_value(json_data,'summary',None)
        json_data = create_key_value(json_data,'keywords',extract_keywords_nlp(text_result))
        if summarizer:
            try:
                json_data['summary'] = summarizer(text_result, max_length=160, min_length=40)
            except Exception as e:
                logging.error(f"Error getting summary for {json_data['filename']}: {e}")
        
        return json_data
    except Exception as e:
        logging.error(f"Error transcribing {audio_path}: {e}")
        return json_data

def get_voice(voice, text=None):
    """Save transcribed text to a file."""
    text = text or ''
    if voice:
        text = text + '\n' + str(voice) if text else str(voice)
       
    return text
def extract_keywords_nlp(strings, top_n=5):

    # Process with spaCy
    doc = nlp(strings)
    
    # Extract nouns, proper nouns, and entities
    word_counts = Counter()
    for token in doc:
        if token.pos_ in ["NOUN", "PROPN"] and not token.is_stop and len(token.text) > 2:
            word_counts[token.text] += 1
    
    # Extract multi-word entities (e.g., "Elon Musk")
    entity_counts = Counter(ent.text.lower() for ent in doc.ents if len(ent.text.split()) > 1)
    
    # Combine and rank
    combined_counts = word_counts + entity_counts
    top_keywords = [word for word, count in combined_counts.most_common(top_n)]
    
    return top_keywords
#!/usr/bin/env python3
import os
import glob
import logging
import json
import sys
import spacy
from datetime import timedelta, datetime
from collections import Counter
from multiprocessing import Process
import fcntl
from transformers import pipeline
from keybert import KeyBERT
import speech_recognition as sr
from moviepy.editor import VideoFileClip
from pydub import AudioSegment
from abstract_utilities import *
from get_video import derive_all_video_meta, analyze_video_text
from transcribe_audio import *
summarizer = pipeline("summarization", model="Falconsai/text_summarization")
keyword_extractor = pipeline("feature-extraction", model="distilbert-base-uncased")
kw_model = KeyBERT(model=keyword_extractor.model)

def get_remove_phrases():
    return ['Video Converter','eeso','Auseesott','Aeseesott','esoft','eeso']
def transcribe_vids(video_path=None):
    video_path = video_path or get_video_dir()
    transcribe_all_video_paths(directory=video_path,output_dir=get_text_dir(),remove_phrases=get_remove_phrases(),summarizer=summarizer,get_vid_data=False)
def get_test_video():
    video_path = '/var/www/typicallyoutliers/frontend/public/repository/Video/Ty Bollinger - CancerTruth.net.mp4'
    transcribe_audio(video_path,output_dir=get_text_dir(),remove_phrases=get_remove_phrases(),summarizer=summarizer,get_vid_data=False)
def get_text_dir():
    text_dir = '/var/www/typicallyoutliers/frontend/public/repository/text_dir'
    return make_dirs(text_dir)
def get_video_dir():
    video_path ='/var/www/typicallyoutliers/frontend/public/repository/Video/'
    return video_path
def get_all_vids():
    transcribe_vids()
processed_files = set()
def check_if_in_time(json_data,start_time,end_time):
    audio_text = json_data.get('audio_text')
    for key,values in audio_text.items():
        value_start_time = values.get("start_time")
        value_end_time = values.get("end_time")
        if value_end_time - value_start_time
def transcribe_audio_file(audio_path, json_data, chunk_length_ms=60000,summarizer=None):
    """Transcribe audio file in chunks and save as text and time-blocked JSON."""
    try:
        logging.info(f"Transcribing audio: {audio_path}")
        audio = AudioSegment.from_wav(audio_path)
        chunks = [audio[i:i + chunk_length_ms] for i in range(0, len(audio), chunk_length_ms)]
        full_text = ""
        json_data = create_key_value(json_data,'audio_text',[])
        file_path = json_data['info_path']
        for i, chunk in enumerate(chunks):
            start_time = i * chunk_length_ms
            end_time = min((i + 1) * chunk_length_ms, len(audio))
            chunk_path = f"chunk_{i}.wav"
            chunk.export(chunk_path, format="wav")
            logging.info(f"Processing chunk {i+1}/{len(chunks)} ({format_timestamp(start_time)} - {format_timestamp(end_time)})")
            
            with sr.AudioFile(chunk_path) as source:
                r.adjust_for_ambient_noise(source)
                audio_data = r.record(source)
                try:
                    text = r.recognize_google(audio_data)
                    full_text += text + " "
                    json_data['audio_text'].append({
                        "start_time": format_timestamp(start_time),
                        "end_time": format_timestamp(end_time),
                        "text": text
                    })
                except sr.UnknownValueError:
                    logging.warning(f"Chunk {i+1} could not be transcribed.")
                    json_data['audio_text'].append({
                        "start_time": format_timestamp(start_time),
                        "end_time": format_timestamp(end_time),
                        "text": ""
                    })
                except sr.RequestError as e:
                    logging.error(f"API error for chunk {i+1}: {e}")
                    json_data['audio_text'].append({
                        "start_time": format_timestamp(start_time),
                        "end_time": format_timestamp(end_time),
                        "text": ""
                    })
                safe_dump_to_file(data=json_data,file_path=json_path) 
            os.remove(chunk_path)  # Clean up chunk file

        # Save plain text
        if full_text:
            full_text = full_text.strip()
        text_result= get_voice(full_text, text=full_text)
        json_data = create_key_value(json_data,'full_text',text_result)
        json_data = create_key_value(json_data,'summary',None)
        json_data = create_key_value(json_data,'keywords',extract_keywords_nlp(text_result))
        if summarizer:
            try:
                json_data['summary'] = summarizer(text_result, max_length=160, min_length=40)
            except Exception as e:
                logging.error(f"Error getting summary for {json_data['filename']}: {e}")
        
        return json_data
    except Exception as e:
        logging.error(f"Error transcribing {audio_path}: {e}")
        return json_data
def get_seo_info(info=None, video_path=None, audio_path=None, output_dir=None, remove_phrases=None, summarizer=None, get_vid_data=False):
    logger.info(f"Entering get_seo_info: {video_path}")
    get_vid_data = get_vid_data or False
    remove_phrases = remove_phrases or []

    video_path = video_path or info.get('video_path')
    audio_path = audio_path or info.get('audio_path')
    video_directory = info.get('info_directory')
    info_path = os.path.join(video_directory, 'info.json')
    dirname = os.path.dirname(video_path)
    basename = os.path.basename(video_path)
    filename, ext = os.path.splitext(basename)

    # Avoid redundant video_text processing
    if not info.get('video_text'):
        logger.info(f"Processing video_text: {video_path}")
        video_text = analyze_video_text(video_path, output_dir=info['thumbnails_dir'], video_text=info.get('video_text'), remove_phrases=remove_phrases)
        info['video_text'] = video_text
    else:
        video_text = info['video_text']

    info.update({
        'video_text': video_text,
        'video_path': video_path,
        'info_directory': video_directory,
        'info_path': info_path,
        'filename': filename,
        'ext': ext,
        'audio_path': os.path.join(video_directory, 'audio.wav'),
        'video_json': os.path.join(video_directory, 'video_json.json')
    })

    if get_vid_data and not os.path.exists(info['video_json']):
        initiate_process(derive_all_video_meta, video_path, video_directory, info['video_json'], [], '', filename, remove_phrases, summarizer)

    if not os.path.isfile(audio_path):
        extract_audio_from_video(video_path=info['video_path'], audio_path=info['audio_path'])

    info = transcribe_audio_file(info['audio_path'], info, chunk_length_ms=60000)

    # Add SEO metadata
    primary_keyword = info['combined_keywords'][0] if info['combined_keywords'] else filename
    info['seo_title'] = f"{primary_keyword} - {filename}"[:70]
    summary = info.get('summary', 'No summary available.')
    keywords_str = ', '.join(info['combined_keywords'][:3])
    info['seo_description'] = f"{summary[:150]} Explore {keywords_str}. Visit thedailydialectics.com for more!"[:300]
    info['seo_tags'] = [kw for kw in info['combined_keywords'] if kw.lower() not in ['video', 'audio', 'file']]
    best_frame, score, matched_text = pick_optimal_thumbnail(info['video_text'], info['combined_keywords']) or ('default.jpg', 0, '')
    info['thumbnail'] = {
        'file_path': os.path.join(info['thumbnails_dir'], best_frame),
        'alt_text': matched_text[:100]
    }
    audio = AudioSegment.from_wav(info['audio_path'])
    info['duration_seconds'] = len(audio) / 1000
    info['duration_formatted'] = format_timestamp(len(audio))
    export_srt(info['audio_text'], f"{video_directory}/captions.srt")
    info['captions_path'] = f"{video_directory}/captions.srt"
    info['schema_markup'] = {
        "@context": "https://schema.org",
        "@type": "VideoObject",
        "name": info['seo_title'],
        "description": info['seo_description'],
        "thumbnailUrl": info['thumbnail']['file_path'],
        "duration": f"PT{int(info['duration_seconds'] // 60)}M{int(info['duration_seconds'] % 60)}S",
        "uploadDate": datetime.now().isoformat(),
        "contentUrl": info['video_path'],
        "keywords": info['seo_tags']
    }
    info['social_metadata'] = {
        "og:title": info['seo_title'],
        "og:description": info['seo_description'],
        "og:image": info['thumbnail']['file_path'],
        "og:video": info['video_path'],
        "twitter:card": "player",
        "twitter:title": info['seo_title'],
        "twitter:description": info['seo_description'],
        "twitter:image": info['thumbnail']['file_path']
    }
    categories = {'ai': 'Technology', 'cannabis': 'Health', 'elon musk': 'Business'}
    info['category'] = next((v for k, v in categories.items() if k in ' '.join(info['seo_tags']).lower()), 'General')
    info['uploader'] = {"name": "The Daily Dialectics", "url": "https://thedailydialectics.com"}
    info['publication_date'] = datetime.now().isoformat()
    video = VideoFileClip(info['video_path'])
    info['file_metadata'] = {
        'resolution': f"{video.w}x{video.h}",
        'format': 'MP4',
        'file_size_mb': os.path.getsize(info['video_path']) / (1024 * 1024)
    }
    video.close()
    video_id = filename.replace(' ', '-').lower()
    info['canonical_url'] = f"https://thedailydialectics.com/videos/{video_id}"
    update_sitemap(info, f"{video_directory}/../sitemap.xml")

    safe_dump_to_file(data=info, file_path=info['info_path'])
    logger.info(f"Exiting get_seo_info: {video_path}")
    return info

def transcribe_all_video_paths(directory=None, output_dir=None, remove_phrases=None, summarizer=None, get_vid_data=None):
    get_vid_data = get_vid_data or False
    remove_phrases = remove_phrases or []
    directory = directory or os.getcwd()
    output_dir = output_dir if output_dir else make_dirs(directory, 'text_dir')
    video_paths = get_all_file_types(directory=directory, types='video')
    for video_path in video_paths:
        if video_path not in processed_files:
            processed_files.add(video_path)
            lock_file = "/tmp/vid_to_aud.lock"
            with open(lock_file, 'w') as f:
                try:
                    fcntl.flock(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
                    transcribe_audio(video_path, output_dir=output_dir, remove_phrases=remove_phrases, summarizer=summarizer, get_vid_data=get_vid_data)
                except IOError:
                    logger.info(f"Another instance is running, skipping: {video_path}")
                finally:
                    fcntl.flock(f, fcntl.LOCK_UN)
def transcribe_vids(video_path=None):
    video_path = video_path or get_video_dir()
    transcribe_all_video_paths(directory=video_path,output_dir=get_text_dir(),remove_phrases=get_remove_phrases(),summarizer=summarizer,get_vid_data=False)
get_all_vids()



# keybert_manager.py

import os
from .keybert_call_schema import get_keyword_execution_variables  # :contentReference[oaicite:0]{index=0}&#8203;:contentReference[oaicite:1]{index=1}
from abstract_utilities import safe_dump_to_file
from .routes import get_bool_response, get_key_vars, get_video_info_path

class KeywordManager:
    """
    Orchestrates:
      1) deciding if keywords need to be run,
      2) pulling input vars,
      3) calling the keyword‐service,
      4) persisting results.
    """
    def fetch_step_vars(self, req=None, info_data=None):
        # returns (inputs_dict, info_data, service_fn, bool_key)
        keys, service_fn, bool_key = get_keyword_execution_variables(
            req=req, info_data=info_data
        )
        inputs, info_data = get_key_vars(keys, req=req, info_data=info_data)
        return inputs, info_data, service_fn, bool_key

    def should_run(self, bool_key, info_data):
        # wraps your boolean‐check helper
        return get_bool_response(bool_key, info_data)

    def run(self, req=None, info_data=None):
        # 1) Gather everything we need
        inputs, info_data, service_fn, bool_key = self.fetch_step_vars(info_data)

        # 2) Decide if we actually extract keywords
        if not self.should_run(bool_key, info_data):
            return info_data
        if 'info_data' not in inputs:
            inputs['info_data'] = info_data
        # 3) Call into keybert_get_data.get_keywords_info_data
        #    which will update info_data['keywords'], ['combined_keywords'], etc.
        updated_info = service_fn(**inputs)
        
        # 4) Persist immediately back to disk
        path = get_video_info_path(**info_data)
        safe_dump_to_file(data=updated_info, file_path=path)

        return updated_info
